/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/**************************************************************************
 *	File:	db_mod.c
 *
 * Corresponds to chapter 8.8: Modifying an Element Instance into 
 * the data base
 *
 **************************************************************************/

#include <sys/types.h>
#include <rpc/types.h>
#include <stdio.h>
#ifdef SVR4
#include <sys/utsname.h>
#endif
#include <netmgt/netmgt_db.h>

main()
{
snmdb_buffer	buf;		/* cluster record buffer of element */
char	local_host[256];	/* local host name */
#ifdef SVR4
struct utsname ut;
#endif

	/* Let's take the local host name as element to search 
	 * in the data base
	 */
#ifdef SVR4
	if (uname(&ut) == -1) {
		perror("uname");
		return 1;
	} else
		strcpy(local_host, ut.nodename);
#else
	if (gethostname(local_host, sizeof(local_host)) == -1) {
		perror("gethostname");
		return 1;
	}
#endif

	/* Open the database */
	if (!snmdb_open()) {
		printf("snmdb_open() failed, error = %d\n", snm_error);
		exit(1);
	}

	/* first, we load the element record into the cluster record buffer */
	if (!snmdb_read("andrew",&buf)) {	/* read error */
		printf("snmdb_read() failed, error = %d\n", snm_error);
		exit(1);
	}

	/* modify the property */
	if (!snmdb_set_property(&buf, "User", "admin machine") ) {
		printf("snmdb_set_property() failed, error = %d\n",
			snm_error);
		exit(1);
	}


	/* delete the connection to another element */
	if (!snmdb_delete_connection(&buf, local_host)) { 
		printf("snmdb_delete_connection() failed, error = %d\n",
			snm_error);
		exit(1);
	}

	/* delete the agent record which describes 'hostif' */
	if (!snmdb_delete_agent(&buf, "hostif")) {
		printf("snmdb_delete_agent() failed, error = %d\n",
			snm_error);
		exit(1);
	}

	/* delete the color record, use default color */
	snmdb_delete_color(&buf);


	/* now we write the update element back to the database */
	snmdb_lock(TRUE, FALSE);
	if (!snmdb_update(&buf)) {
		printf("snmdb_update() failed, error = %d\n", snm_error);
		snmdb_unlock();
		exit(1);
	}

	snmdb_unlock();
	printf("element 'andrew' is updated succesfully \n");

}

