/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/**************************************************************************
 *	File:	event_diplay.c
 *
 * Corresponds to chapter 2.7:  "Registering for Data, Event, and Trap Reports"
 *                        3.7:  "Getting Data, Event and Trap Reports"
 *
 **************************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netmgt/netmgt.h>

#include <signal.h>


char 		myname[MAXHOSTNAMELEN];
int		sock_udp, sock_tcp;
u_long		rendez;
struct timeval	timeout = {NETMGT_TIMEOUT, 0};


void		report_handler();
void 		print_agent_error();
void		my_exit();
void		show_trap();
void		show_event();
#ifdef SVR4
char * 		value2str(u_int type, caddr_t val, u_int len);
char * 		prio2str(u_int pri);
char *		relop2str(u_int op);
#else
char *		value2str();
char *		prio2str();
char *          relop2str();
#endif


/* exit reasons */
#define INTERRUPT	0
#define CALLBACK_ERR	1
#define RENDEZ_ERR	2
#define NO_EVENT	3
#define LAST_REPORT	4

static void
#ifdef SVR4
sigint(int i)
#else
sigint()
#endif
{
	my_exit(INTERRUPT);
}

/* 
 * print_agent_error
 * see Chapter 6 of the Programmer's Guide : Handling Error Reports
 */

void print_agent_error()
{
Netmgt_error 	errbuf;
char 		agentMsg[NETMGT_NAMESIZ];

	(void) netmgt_fetch_error(&errbuf);

	if ((int)errbuf.service_error > (int)NETMGT_FATAL ) {
		printf( "Generic error: %s, %s\n",
		    netmgt_sperror(), errbuf.message);
	} else {

		printf( "Agent %s: %d, %s\n",
		    (errbuf.service_error==NETMGT_FATAL) ? "error" : "warning",
		    errbuf.agent_error,errbuf.message);

	}
}



/* 
 * value2str: 
 *	returns a printable string for the value of attribute given as input
 */


#ifdef SVR4
char * value2str(u_int type, caddr_t val, u_int len)
#else
char * value2str(type, val, len)
u_int type;
caddr_t val; 
u_int len;
#endif
{

char 		buf[256];
struct in_addr 	inaddr;
u_int		i;

	switch (type) {

	case NETMGT_SHORT:
		(void) sprintf(buf, "%d", *(short *) val);
		break;

	case NETMGT_U_SHORT:
		(void) sprintf(buf, "%u", *(u_short *) val);
		break;

	case NETMGT_INT:
	case NETMGT_LONG:
	case NETMGT_INTEGER: 
		(void) sprintf(buf, "%d", *(int *) val);
		break;

	case NETMGT_FLOAT:
		(void) sprintf(buf, "%f", *(float *) val);
		break;

	case NETMGT_DOUBLE:
		(void) sprintf(buf, "%f", *(double *) val);
		break;

	case NETMGT_U_INT:
	case NETMGT_U_LONG:
	case NETMGT_COUNTER:
	case NETMGT_GAUGE:
	case NETMGT_TIMETICKS:
		(void) sprintf(buf, "%u", *(u_int *) val);
		break;

	case NETMGT_OBJECT_IDENTIFIER:
		(void) sprintf(buf, "%s", (char *) netmgt_oid2string(len, val));
		break;

	case NETMGT_NET_ADDRESS:
	case NETMGT_IP_ADDRESS:
		inaddr.s_addr = ntohl(*(u_long *) val);
		(void) sprintf(buf, "%s", inet_ntoa(inaddr));
		break;

	case NETMGT_INADDR:
		(void) sprintf(buf, "%s", inet_ntoa(*(struct in_addr *) val));
        	break;
       
	case NETMGT_STRING:
		return (val);

	case NETMGT_UNIXTIME:
		(void) sprintf(buf, "%s", (char *) ctime( (time_t *)val));
		break;

	case NETMGT_TIMEVAL:
		(void) sprintf(buf, "%d.%d",
		    ((struct timeval *) val)->tv_sec,
		    ((struct timeval *) val)->tv_usec);
		break;

	case NETMGT_OCTET:
	case NETMGT_OCTET_STRING: {
		char buf[8192 * 2];
		for (i = 0; i < len; i++)
		    (void) sprintf(&buf[i * 2], "%02x", (u_char)val[i]);
		return buf;
    	}


    	case NETMGT_ENUM:
		/* should get enum value from attribute list */
		(void) sprintf(buf, "%d", *(int *) val);
		break;

    	default:
		return ((char *) NULL);
	}
	return (buf);
}

static char *relop[] = {
"No Operator",			/* no operation */
"Equal To",			/* value == threshold */
"Not Equal To",			/* value != threshold */
"Less Than",			/* value <  threshold */
"Less Than Or Equal To ",	/* value <= threshold */
"Greater Than ",		/* value >  threshold */
"Greater Than Or Equal To ",	/* value >= threshold */
"Changed ",			/* value changed */
"Increased By ",		/* value increased by threshold */
"Decreased By ",		/* value decreased by threshold */
"Increased By More Than ",	/* value increased by more than threshold */
"Increased By Less Than ",	/* value increased by less than threshold */
"Decreased By More Than ",	/* value decreased by more than threshold */
"Decreased By Less Than ",	/* value decreased by less than threshold */
"Trap ",			/* trap */
};

/*
 * relop2str:
 * 	returns a printable string for the operator given as parameter
 */

#ifdef SVR4
char *relop2str(u_int op)
#else
char *relop2str(op)
u_int op;
#endif
{
	if (op > (u_int)NETMGT_TRAP)
		return("unknown operator");
	else
		return(relop[op]);
}

char *prio[] = {
"NO_PRIORITY",
"LOW",			/* low priority */
"MEDIUM",		/* medium priority */
"HIGH",			/* high priority */
};


/*
 * prio2str:
 * 	returns a printable string for the priority given as parameter
 */

#ifdef SVR4
char * prio2str(u_int pri)
#else
char * prio2str(pri)
u_int pri;
#endif
{
	if (pri > (u_int)NETMGT_HIGH_PRIORITY)
		return("unknown priority");
	else
		return(prio[pri]);
}

#ifdef SVR4
void my_exit(u_int when)
#else
void my_exit(when)
u_int when;
#endif
{

	if (!netmgt_unregister_rendez(myname, myname, rendez, 
	    NETMGT_VERS, NETMGT_ANY_AGENT, NETMGT_LOW_PRIORITY, 
	    timeout)) {
		(void) printf("netmgt_unregister_rendez failed : %s \n",
		    netmgt_sperror()); 
	}

	netmgt_unregister_callback(rendez, NETMGT_VERS);
	exit(when);
}


#ifdef SVR4
void report_handler(u_int type, char *target, char *group, char *key,
	u_int count, struct timeval interval, u_int flags)
#else
void report_handler(type, target, group, key, count, interval, flags)
u_int type;
char *target;
char *group;
char *key;
u_int count;
struct timeval interval;
u_int flags;
#endif
{
Netmgt_msginfo	msginfo;		/* report message information */

	printf("report_handler: called\n");

	/* Get additional report message information. */
	if (!netmgt_fetch_msginfo(&msginfo)) {
		NETMGT_DBG("can't fetch message information: %s\n",
		    netmgt_sperror());
		return;
	}

	printf("IP address of the requesting manager : %s\n", 
		inet_ntoa(msginfo.manager_addr));
	printf("IP address of the responding agent: %s\n", 
		inet_ntoa(msginfo.agent_addr));


	if (flags & NETMGT_NO_EVENTS) {
		(void) printf("(last report from agent [got NETMGT_NO_EVENTS])\n");
		my_exit(NO_EVENT);
	}


	switch (type) {
	case NETMGT_ERROR_REPORT :
		print_agent_error();
		break;

	case NETMGT_DATA_REPORT:
		printf("report handler: data report\n");
		break;

	case NETMGT_TRAP_REPORT:
		printf("report handler: trap report\n");
		show_trap();
		break;

	case NETMGT_EVENT_REPORT:
		printf("report handler: event report\n");
		show_event();
		break;

	default:
		printf("report handler: unknown report\n");

	}

	if (flags & NETMGT_LAST) {
		(void) printf ("(last report from agent) \n");
		my_exit(LAST_REPORT);
 	}

}


void
show_trap()
{
Netmgt_data	trap;			/* data-report buffer */
char *s;


	strcpy(trap.name, "!a!");
	while (strcmp(trap.name, NETMGT_ENDOFARGS)) {
		if (!netmgt_fetch_data(&trap)) {
			NETMGT_DBG1("can't fetch trap statistics\n");
			print_agent_error();
		}

		if (!strcmp(trap.name, NETMGT_ENDOFROW)) {
			(void) printf("(end of row) \n\n");
			continue;
		}

		if (!strcmp(trap.name, NETMGT_ENDOFARGS)) {
			(void) printf( "(end of report) \n\n");
			continue;
		}

		/* print the received information */
		if (s = value2str(trap.type, trap.value, trap.length)) {
			(void) printf ("%s = %s", trap.name, s);
		} else {
			(void) printf("%s : has an UNKNOWN type \n",
			    trap.name, trap.type);
		}

		(void) printf("\n");
	}
}




void
show_event()
{
Netmgt_event	event;			/* event-report buffer */
char *s;

	strcpy(event.name, "!a!");
	while (strcmp(event.name, NETMGT_ENDOFARGS)) {
		if (!netmgt_fetch_event(&event)) {
			NETMGT_DBG1("can't fetch event statistics\n");
			print_agent_error();
		}

		if (!strcmp(event.name, NETMGT_ENDOFROW)) {
			(void) printf("end of row) \n\n");
			continue;
		}

		if (!strcmp(event.name, NETMGT_ENDOFARGS)) {
			(void) printf( "(end of report) \n\n");
			continue;
		}

		/* print the received information */
		if (s = value2str(event.type, event.value, event.length)) {
			(void) printf ("%s = %s", event.name, s);

			if (event.relop != NETMGT_NOP) {
				s = value2str(event.type, event.thresh_val, 
				    event.length);
				(void) printf("   (%s %s Priority %s)",
				    relop2str(event.relop), s, 
				    prio2str(event.priority));
			}
		} else {
			(void) printf("%s : has an UNKNOWN type \n",
			    event.name, event.type);
		}

		(void) printf("\n");
	}

}

#ifdef SVR4
main(int argc, char *argv[])
#else
main(argc, argv)
int argc;
char *argv[];
#endif

{
	(void) signal(SIGINT, sigint);
	(void) signal(SIGTERM, sigint);


	/* get my hostname */
	gethostname(myname, MAXHOSTNAMELEN);

	/* Create a transient RPC program number using any UDP and TCP socket*/
	sock_udp = RPC_ANYSOCK;
	sock_tcp = RPC_ANYSOCK;

	/* Register the function "report_handler " as the function to be called
	 * for any report sent to the RPC program number returned in variable
	 * "rendez"
	 */
	if (!(rendez = netmgt_register_callback(report_handler, &sock_udp,
		&sock_tcp, NETMGT_VERS, IPPROTO_UDP | IPPROTO_TCP))) {

		(void) fprintf(stderr, "Can't register callback: %s\n",
				netmgt_sperror());
		exit(CALLBACK_ERR);
	}

	/* Register the program identified by "rendez" with the local 
	 * event dispatcher in so the function "report_handler" will get 
	 * event reports too
	 */
	if (!netmgt_register_rendez(myname, myname, rendez, NETMGT_VERS,
		NETMGT_ANY_AGENT, NETMGT_LOW_PRIORITY, timeout)) {

		(void) fprintf(stderr,
			"Can't register with the event dispatcher: %s\n",
			netmgt_sperror());

		netmgt_unregister_callback(rendez, NETMGT_VERS);
		exit(RENDEZ_ERR);
	}

	printf("%s: starting\n", argv[0]);

	/* wait for the reports to come in */
	svc_run();
}
