/* 
 * netmgtCmd.c - Interface snmdb command to Tcl.
 * 
 * Author:	Damir Delija
 * Date:	Fri Aug 19 1996
 */

#include <stdio.h>
#include <tcl.h>


#include <netmgt/netmgt_db.h>

/* ***************************************************************** */

/* ***************************************************************** */

/*aditional parse structure */
typedef  struct {
    char *cmd;			/* command name */
    int (*f)();			/* exec function */
    int minargs, maxargs;	/* number of args min and max */
    int subcmds;
    char *usage;		/* help message */
} cmdOptions;

static cmdOptions subcmdVec[] = {
/*
{"delete_from_view", tclsnmdb_delete_from_view, 2, 2, 0, "buffer view"},
*/

};


/* ################################################################## */
/*
 *
*/
int
netMgtCmd( ClientData clientData, Tcl_Interp *interp, int argc, char *argv[] )
{
    int argi, subi;
    int res;
    int local_snm_error=0;  	
    /* Check for subcommand. */
    if ( argc < 2 )
    {
	interp->result = "wrong # args: should be \"snmdb option ...\"";
	return TCL_ERROR;
    }
    
    /* Find the subcommand. */
    for (subi = 0; subi < (sizeof subcmdVec) / (sizeof subcmdVec[0]); subi++)
    {
	if ( strcmp( subcmdVec[subi].cmd, argv[1] ) == 0 )
	{
	    /* Check arg count. */
	    if ( argc - 2 < subcmdVec[subi].minargs ||
		 argc - 2 > subcmdVec[subi].maxargs )
	    {
		Tcl_AppendResult( interp, "wrong # args: should be \"snmdb ",
				  subcmdVec[subi].cmd, " ",
				  subcmdVec[subi].usage, "\"", 0 );
		return TCL_ERROR;
	    }
	    
	    /* Call the subcommand function. */

    	    local_snm_error=snm_error;  	
	    snm_error=0;
	    res=(*subcmdVec[subi].f)(  clientData, interp, argc-1, argv+1 ); 
	    if (res!=TCL_OK) {
		snmTclAddError (interp,  snm_error );
	    }
      	    snm_error=local_snm_error;
	    /* 
		here will come Error interpretation routine!
	    */

		return res;
	}
    }

    /* If we get here, the option doesn't match. */
    Tcl_AppendResult( interp, "bad option \"", argv[1], "\": should be ", 0 );
    for (subi = 0; subi < (sizeof subcmdVec) / (sizeof subcmdVec[0]); subi++)
	Tcl_AppendResult( interp, (subi > 0 ? ", " : ""),
			   subcmdVec[subi].cmd, 0 );

    return TCL_ERROR;
}

/* ################################################################## */

/* Initialize the package.  */

void Snmdbtcl_Init( Tcl_Interp *interp ) 
{
	if (tableInit(
		&bufferHandleTable,"bfdb", sizeof(snmdb_handle)
		)==TCL_ERROR) { return; }

	if (tableInit(
		&enumHandleTable,"enumdb", sizeof(snmdb_handle)
		)==TCL_ERROR) { return; }

	Tcl_CreateCommand( interp, "snmdb", snmdbCmd,
                       (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL );
}

/* ################################################################## */
