/* Copyright 1994 Sun Microsystems, Inc. All Rights Reserved. */
/* Copyright 1993 Sun Microsystems, Inc. All Rights Reserved. */
/**************************************************************************
 *  File:	netmgt_db.h
 *
 *  Copyright (c) 1991,1992 Sun Microsystems, Inc.  All Rights Reserved.
 *  Sun considers its source code as an unpublished, proprietary trade 
 *  secret, and it is available only under strict license provisions.  
 *  This copyright notice is placed here only to protect Sun in the event
 *  the source is deemed a published work.  Dissassembly, decompilation, 
 *  or other means of reducing the object code to human readable form is 
 *  prohibited by the license agreement under which this code is provided
 *  to the user or company in possession of this copy.
 * 
 *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the 
 *  Government is subject to restrictions as set forth in subparagraph 
 *  (c)(1)(ii) of the Rights in Technical Data and Computer Software 
 *  clause at DFARS 52.227-7013 and in similar clauses in the FAR and 
 *  NASA FAR Supplement.
 *
 *  SCCSID:	@(#)netmgt_db.h 2.30 94/11/30 
 *
 *  Comments:	SunNet Manager Runtime Database definitions
 *
 **************************************************************************
 */

#ifndef _netmgt_db_h
#define _netmgt_db_h

#include <stdio.h>
#define SNMBUFFERSIZE	8192

typedef struct {
    int data[SNMBUFFERSIZE/sizeof(int)];
} snmdb_buffer;

typedef union {
    int db_int;			/* Type = int, enum */
    unsigned int db_uint;	/* Type = unsigned int, counter, gauge */
    long db_long;		/* Type = long */
    unsigned long db_ulong;	/* Type = unsigned long, unixtime, timeticks */
    float db_float;		/* Type = float */
    double db_double;		/* Type = double */
    char *db_string;		/* Type = string, octet */
    short db_short;		/* Type = short */
    unsigned short db_ushort;	/* Type = unsigned short */
    char db_address[16];	/* Type = netaddress, ipaddress */
}snmdb_data;

typedef enum {
	undefined,
	short_integer,
	unsigned_short_integer,
	integer,
	unsigned_integer,
	long_integer,
	unsigned_long_integer,
	floating_point,
	double_floating_point,
	octet,
	string,
	counter,
	gauge,
	timeticks,
	netaddress,
	ipaddress,
	unixtime,
	enumeration
} snmdb_type;

typedef char *snmdb_handle;

/* Errors */

extern int snm_error;

#define SNMDB_BAD_ARGUMENT			1
#define SNMDB_NOT_INITIALIZED			2
#define SNMDB_UNKNOWN_AGENT			3
#define SNMDB_NO_MEMORY				4
#define SNMDB_INVALID_TYPE			5
#define SNMDB_TOO_LONG				6
#define SNMDB_ELEMENT_NOT_FOUND			7
#define SNMDB_NO_KEY_FIELD			8
#define SNMDB_DUPLICATE_ID			9
#define SNMDB_DUPLICATE_NAME			10
#define SNMDB_CONNECTED_TO_ELEMENT_NOT_FOUND	11
#define SNMDB_CONNECTION_EXISTS		        12
#define SNMDB_CONNECTION_NOT_FOUND		13
#define SNMDB_CANNOT_CONNECT_TO_SELF		14
#define SNMDB_ELEMENT_ALREADY_IN_VIEW		15
#define SNMDB_ELEMENT_NOT_IN_VIEW		16
#define SNMDB_SUBVIEW_IS_NOT_EMPTY		17
#define SNMDB_UNABLE_TO_DELETE_ELEMENT		18
#define SNMDB_UNKNOWN_PROPERTY		        19
#define SNMDB_INVALID_DATA_IN_BUFFER	        20
#define SNMDB_NOT_CONNECTED_TO_ANY              21
#define SNMDB_NOT_IN_ANY_VIEW		        22
#define SNMDB_NO_AGENT_APPLY     	        23
#define SNMDB_AGENT_NOT_FOUND     	        24
#define SNMDB_X_AND_Y_UNDEFINED     	        25
#define SNMDB_INVALID_FILE      	        26
#define SNMDB_GLYPH_FILENAME_NOT_FOUND  	27
#define SNMDB_ALIAS_NOT_FOUND       	        28
#define SNMDB_NO_ALIAS_EXIST                    29
#define SNMDB_UNABLE_TO_DELETE_ALIAS		30

/* ancillary agent RPC program numbers (also defined in netmgt_rpc.h) */
#ifndef NETMGT_DBMGR_PROG
#define NETMGT_DBMGR_PROG       (u_long)100110	/* database manager */
#define NETMGT_DBMGR_VERS       NETMGT_VERS
#endif


/* package public functions with C linkage */
#if defined (__cplusplus)
extern "C" {
#endif

/* For all function declarations, if ANSI then use a prototype */
#if  defined (__STDC__)

/* Package public database services */
int snmdb_add (snmdb_buffer *);
int snmdb_add_agent	(snmdb_buffer *, char *, char *);
int snmdb_add_connection (snmdb_buffer *, char *);
int snmdb_add_alias      (snmdb_buffer *, char *);
int snmdb_add_to_view (snmdb_buffer *, char *, int, int, int, ...);
int snmdb_delete (char *);
int snmdb_delete_agent (snmdb_buffer *, char *);
int snmdb_delete_color	(snmdb_buffer *);
int snmdb_delete_connection	(snmdb_buffer *, char *);
int snmdb_delete_alias  	(snmdb_buffer *, char *);
int snmdb_delete_from_view	(snmdb_buffer *, char *);
char **snmdb_enumerate_agents (snmdb_buffer *);
char **snmdb_enumerate_aliases     (snmdb_buffer *);
char **snmdb_enumerate_connections (snmdb_buffer *);
snmdb_handle *snmdb_enumerate_elements	(char *, char *);
char **snmdb_enumerate_views (snmdb_buffer *);
void snmdb_free_enumeration_handle	(snmdb_handle *);
void snmdb_free_list (char **);
int snmdb_get_agent	(snmdb_buffer *, char *, int *, char *);
int snmdb_get_color	(snmdb_buffer *, int *, int *, int *);
char *snmdb_get_element_glyph (char *);
char *snmdb_get_element_type (snmdb_buffer *);
char *snmdb_get_next_element (snmdb_handle *);
snmdb_data snmdb_get_property (snmdb_buffer *, char *, snmdb_type *);
int snmdb_get_view (snmdb_buffer *, char *, int *, int *, int *, ...); 
/* int snmdb_get_view (); */
int snmdb_init_buffer (char *, char *, snmdb_buffer *);
int snmdb_lock (int, int);
int snmdb_open ();
int snmdb_read (char *, snmdb_buffer *);
int snmdb_save_element (snmdb_buffer *, FILE *);
int snmdb_set_color (snmdb_buffer *, int, int, int);
int snmdb_set_property (snmdb_buffer *, char *, ...);
int snmdb_unlock ();
int snmdb_update (snmdb_buffer *);

/* Public Console API functions -- only work when the Console is running */
int snmdb_console_load			(char *, int *, int *);
int snmdb_console_reload		(char *, int *, int *);
int snmdb_console_save_components	(char *, int *, int *);

/* Not ANSI C, don't use prototypes */
#else

int snmdb_add			();
int snmdb_add_agent		();
int snmdb_add_connection	();
int snmdb_add_alias		();
int snmdb_add_to_view		();
int snmdb_delete		();
int snmdb_delete_agent		();
int snmdb_delete_color		();
int snmdb_delete_connection	();
int snmdb_delete_alias		();
int snmdb_delete_from_view	();
char **snmdb_enumerate_agents		();
char **snmdb_enumerate_connections	();
char **snmdb_enumerate_aliases          ();
snmdb_handle *snmdb_enumerate_elements	();
char **snmdb_enumerate_views		();
void snmdb_free_enumeration_handle	();
void snmdb_free_list		();
int snmdb_get_agent		();
int snmdb_get_color		();
char *snmdb_get_element_glyph	();
char *snmdb_get_element_type	();
char *snmdb_get_next_element	();
snmdb_data snmdb_get_property	();
int snmdb_get_view		();
int snmdb_init_buffer		();
int snmdb_lock			();
int snmdb_open			();
int snmdb_read			();
int snmdb_save_element		();
int snmdb_set_color		();
int snmdb_set_property		();
int snmdb_unlock		();
int snmdb_update		();

/* Public Console API functions -- only work when the Console is running */
int snmdb_console_load			();
int snmdb_console_reload		();
int snmdb_console_save_components	();


#endif

#ifdef __cplusplus
}
#endif
 
#endif /* !_netmgt_db_h */
