/* 
 * gdCmd.c - Interface gd to Tcl.
 * 
 * Author:	Spencer W. Thomas
 * 		Information Technology and Networking
 * 		University of Michigan
 * Date:	Fri Aug 19 1994
 * Copyright (c) 1994, University of Michigan
 */

/* #define TCLX_SUPPORT */

#include <stdio.h>
#include <tcl.h>
#include "handle_t.h"



/*****************************************************************
 * Simulate TclX Handle routines using hash tables.
 * 
 */
void_pt Tcl_HandleTblInit(const char *handleBase, int entrySize,
			  int initEntries)
{
    HandleTable *ht;

    ht = (HandleTable *)calloc(1, sizeof(HandleTable));
    if (ht == 0) {
	return (void_pt)0;
    }
    Tcl_InitHashTable(&ht->theTable, TCL_STRING_KEYS);
    ht->entrySize = entrySize;
    ht->cnt=0;
    if ((ht->handleBase = (char *)calloc(strlen(handleBase)+1, 1)) == 0) {
	free(ht);
	return (void_pt)0;
    }
    strcpy(ht->handleBase, handleBase);

    return (void_pt)ht;
}

void_pt Tcl_HandleAlloc(void_pt headerPtr, char *handlePtr)
{
    HandleTable *hp;
    int n, new;
    Tcl_HashEntry *p;
    void_pt *data;

    if (headerPtr==NULL) return NULL;
    hp = (HandleTable *)headerPtr;
    /* First create a new handle name. */
    n = 0;
/*    n=hp->cnt++; */
    do {
	sprintf(handlePtr, "%s%u", hp->handleBase, n);
	n++;
	p = Tcl_CreateHashEntry(&hp->theTable, handlePtr, &new);
    } while (!new);

    data = (void_pt) calloc(1, hp->entrySize);
    Tcl_SetHashValue( p, data);
    
    return data;
}

void_pt Tcl_HandleXlate(Tcl_Interp *interp, void_pt headerPtr,
			const char *handle)
{
    Tcl_HashEntry *ep;
    HandleTable *hp;

    if (headerPtr==NULL) return NULL;
    hp = (HandleTable *)headerPtr;

    ep = Tcl_FindHashEntry(&hp->theTable, handle);
    if (ep == NULL) {
	Tcl_AppendResult(interp, "No handle named \"", handle, "\"", NULL);
	return NULL;
    }
    return (void_pt)Tcl_GetHashValue(ep);
}

void Tcl_HandleFree(void_pt headerPtr, void_pt entryPtr)
{
    HandleTable *hp;
    Tcl_HashEntry *ep;
    Tcl_HashSearch srch;
    void_pt data;

    if (headerPtr==NULL) return ;
    hp = (HandleTable *)headerPtr;
    /* Have to search the hash table to find an entry equal to the pointer. */
    for (ep = Tcl_FirstHashEntry(&hp->theTable, &srch);
	 ep != NULL && entryPtr != Tcl_GetHashValue(ep);
	 ep = Tcl_NextHashEntry(&srch))
	;
    if (ep == NULL)
	return;

    /* Found it, now delete it. */
    Tcl_DeleteHashEntry(ep);
    free(entryPtr);
}

/*
find data in table by it poinyter
*/
void_pt Tcl_HandlePtrFind(void_pt headerPtr, void_pt entryPtr)
{
    HandleTable *hp;
    Tcl_HashEntry *ep;
    Tcl_HashSearch srch;
    void_pt data;
    if (headerPtr==NULL) return NULL;
    if (entryPtr==NULL) return NULL;

    hp = (HandleTable *)headerPtr;
    /* Have to search the hash table to find an entry equal to the pointer. */
    for (ep = Tcl_FirstHashEntry(&hp->theTable, &srch);
	 ep != NULL && entryPtr != Tcl_GetHashValue(ep);
	 ep = Tcl_NextHashEntry(&srch));

	return ep;
}

/*
for tables if name exist than name is used as key for hashing,
else key is normaly generated
*/
void_pt Tcl_HandleAllocName(void_pt headerPtr, char *handlePtr, char *name)
{
    HandleTable *hp;
    int n, new;
    Tcl_HashEntry *p;
    void_pt *data;
    char *handleBase;

    if (headerPtr==NULL) return NULL;
    hp = (HandleTable *)headerPtr;

	if (name!=NULL && *name ) handleBase=name;
	else handleBase=hp->handleBase;

    /* First create a new handle name. */
    n = -1;
/*    n=hp->cnt++; */
    do {
	if (n==-1) {
		strcpy( handlePtr, handleBase);
	} else {
		sprintf(handlePtr, "%s%u", handleBase, n);
	}
	n++;
	p = Tcl_CreateHashEntry(&hp->theTable, handlePtr, &new);
    } while (!new);

    data = (void_pt) calloc(1, hp->entrySize);
    Tcl_SetHashValue( p, data);
    
    return data;
}

int tableInit ( void_pt *table, char *name, int size )
{
 
    if ( *table == 0 )
    {
      *table= Tcl_HandleTblInit( name, size, 1 );
      if ( *table == 0 )
      {
            return TCL_ERROR;
      }
    }
        return TCL_OK;
}
 
/* 
end
*/
